var _, fs, generateDirectory, generateFile, generateTree, gutil, path, success, templateBasePath, templates, tree;

fs = require("fs");

path = require("path");

_ = require("lodash");

gutil = require("gulp-util");

tree = require('./tree.js');

templateBasePath = path.join("..", "templates", "html");

templates = {
  directory: _.template(fs.readFileSync(path.resolve(__dirname, path.join(templateBasePath, "directory.underscore")))),
  file: _.template(fs.readFileSync(path.resolve(__dirname, path.join(templateBasePath, "file.underscore"))))
};

success = function(percentage) {
  if (percentage === -1 || percentage >= 90) {
    return "pass";
  } else if (percentage >= 70) {
    return "warn";
  } else {
    return "fail";
  }
};

generateTree = function(suiteData, tree, curPath) {
  var child, generated, name, ref;
  if (curPath == null) {
    curPath = [];
  }
  if (tree.type === 'file') {
    curPath.push(tree.name);
    generated = generateFile(suiteData, tree, curPath);
    curPath.pop();
  } else {
    generated = [generateDirectory(suiteData, tree, curPath)];
    ref = tree.children;
    for (name in ref) {
      child = ref[name];
      curPath.push(tree.name);
      generated = generated.concat(generateTree(suiteData, child, curPath));
      curPath.pop();
    }
  }
  return generated;
};

generateFile = function(suiteData, info, curPath) {
  var date, html, source;
  if (curPath == null) {
    curPath = [];
  }
  source = null;
  if (fs.statSync(info.path).isFile()) {
    source = fs.readFileSync(info.path, {
      encoding: 'utf8'
    }).split('\n');
  }
  date = new Date();
  html = templates.file({
    success: success,
    path: curPath,
    suite: suiteData.name,
    date: suiteData.date,
    info: info,
    source: source
  });
  return new gutil.File({
    path: curPath.slice(1).join(path.sep) + ".html",
    contents: new Buffer(html)
  });
};

generateDirectory = function(suiteData, info, curPath) {
  var child, date, directoryPath, files, html, name, ref, subdirectories;
  if (curPath == null) {
    curPath = [];
  }
  directoryPath = curPath;
  if (info.name !== '') {
    directoryPath = curPath.concat(info.name);
  }
  subdirectories = [];
  files = [];
  ref = info.children;
  for (name in ref) {
    child = ref[name];
    if (child.type === 'file') {
      files.push(child);
    } else {
      subdirectories.push(child);
    }
  }
  date = new Date();
  html = templates.directory({
    success: success,
    path: directoryPath,
    suite: suiteData.name,
    date: suiteData.date,
    info: info,
    subdirectories: subdirectories,
    files: files
  });
  return new gutil.File({
    path: directoryPath.slice(1).concat('index.html').join(path.sep),
    contents: new Buffer(html)
  });
};

module.exports = function(suiteData) {
  return generateTree(suiteData, tree(suiteData.files));
};
